@@ -104,3 +104,10 @@ assert_values2 <- function(nX, values, values2, fcn, debug = FALSE) {
     stop(ex)
   }
 }
+
+
+stealth_sample.int <- function(n, size = n, replace = FALSE, ...) {
+  oseed <- .GlobalEnv$.Random.seed
+  on.exit(.GlobalEnv$.Random.seed <- oseed)
+  sample.int(n = n, size = size, replace = replace, ...)
+}
